@@ -63,6 +63,11 @@ def test_origin():
     assert URL("http://example.com:8888") == url.origin()
 
 
+def test_origin_with_no_auth():
+    url = URL("http://example.com:8888/path/to?a=1&b=2")
+    assert URL("http://example.com:8888") == url.origin()
+
+
 def test_origin_nonascii():
     url = URL("http://user:password@оун-упа.укр:8888/path/to?a=1&b=2")
     assert str(url.origin()) == "http://xn----8sb1bdhvc.xn--j1amh:8888"
@@ -304,11 +309,13 @@ def test_port_for_implicit_port():
 def test_port_for_relative_url():
     url = URL("/path/to")
     assert url.port is None
+    assert url.explicit_port is None
 
 
 def test_port_for_unknown_scheme():
     url = URL("unknown://example.com")
     assert url.port is None
+    assert url.explicit_port is None
 
 
 def test_explicit_port_for_explicit_port():
@@ -948,6 +955,16 @@ def test_joinpath(base, to_join, expected):
         pytest.param("path", "a/", "path/a/", id="default_trailing-empty-segment"),
         pytest.param("path", "a//", "path/a//", id="default_trailing-empty-segments"),
         pytest.param("path", "a//b", "path/a//b", id="default_embedded-empty-segment"),
+        pytest.param(
+            "path/a/b/c/d/e", "a/../../../../../../c", "path/c", id="long-backtrack"
+        ),
+        pytest.param(
+            "path/a/b/c/d/e",
+            "a/../../../././../../../c",
+            "path/c",
+            id="long-backtrack-with-dots",
+        ),
+        pytest.param("path/a/../../d/e", "a/../c", "d/e/c", id="backtrack-in-both"),
     ],
 )
 def test_joinpath_empty_segments(base, to_join, expected):
@@ -958,6 +975,14 @@ def test_joinpath_empty_segments(base, to_join, expected):
     )
 
 
+def test_joinpath_backtrack_to_base():
+    url = URL("http://example.com/../../c")
+    new_url = url.joinpath("../../..")
+    assert str(new_url) == "http://example.com"
+    assert new_url.path == "/"
+    assert new_url.raw_path == "/"
+
+
 def test_joinpath_single_empty_segments():
     """joining standalone empty segments does not create empty segments"""
     a = URL("/1//2///3")
@@ -1473,6 +1498,14 @@ def test_is_default_port_for_unknown_scheme():
     assert not url.is_default_port()
 
 
+def test_handling_port_zero():
+    url = URL("http://example.com:0")
+    assert url.explicit_port == 0
+    assert url.explicit_port == url._val.port
+    assert str(url) == "http://example.com:0"
+    assert not url.is_default_port()
+
+
 #
 
 
